/*
 *  toggle_comment : Sample SEDIT filter
 *
 *    Receives a set of characters separated by new-lines on standard input
 *    and returns then after treatment on the standard output
 *
 *    If the lines received do no start with  /*, they will be commented,
 *    else they will be commented out.
 *
 *  Compile with:
 *
 *     cc -s -Bstatic -O -o toggle_comment toggle_comment.c  -lc -lm
 */

#include <stdio.h>
#include <sgtty.h>          /*   */
#include <values.h>
#include <sys/param.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/file.h>

#include <sys/time.h>   /* usleep() */
#include <signal.h>

#define LIM 100

#define DEBUG 0

char *malloc(), *realloc(), *alloc_n();
void free_n();


void usleep(us)
   unsigned long us;
{
  struct timeval timeout;

  timeout.tv_usec = us % (unsigned long) 1000000;
  timeout.tv_sec = us / (unsigned long) 1000000;

  select(NULL, NULL, NULL, NULL, &timeout);
}

main()
{
   int rho[2], *lens, iin, nl, lgcom;
   char **tbu;
   register i, j, k;

#if DEBUG

   if ( -1 == (iin = open("toggle_comment.test", O_RDONLY)) )
    { fprintf(stderr, "filter: unable to open toggle_comment.test\n");
      exit(1);
    }

#else

  /*
   *  We test stdin
   */
   iin = fileno(stdin);

   for(i=0;; i++)
   { ioctl(iin, FIONREAD, &nl);
     if (nl) break;
     usleep(1000);
     if ( i == LIM )
      { fprintf(stderr, "filter: Input empty\n");
        exit(1);
      }
   }

#endif

  readln2( iin, MAXINT, &tbu, rho, &lens );

  /*
   *  Test if first none blank == /*
   */
  lgcom = 0;
  for ( i=0; i<rho[0]; i++ )
   for ( k=0; k<lens[i]-1; k++ )
    if ( tbu[i][k] != ' ' )
     { lgcom = !bcmp(&tbu[i][k], "/*", 2);

       if ( i == 0 )
        { for(j=0; j<k; j++) putc(' ', stdout);
          tbu[0] += k;
        }

       goto end_i;
     }
  end_i:;

  if(lgcom)
   { /*
      * We suppress starting and ending comments delimiters and replace
      * * \ by  * /
      */
      bcopy(tbu[i]+2, tbu[i], lens[i]+1-(2));
      lens[i] -= 2;
      if (tbu[i][0] == ' ')
       { bcopy(tbu[i]+1, tbu[i], lens[i]+1);
         -- lens[i];
       }

      for ( i=0; i<rho[0]; i++ )
        for ( k=0; k<lens[i]-1; k++ )
         { if (!bcmp(&tbu[i][k], "*\\", 2)) tbu[i][k+1] = '/';
           if (!bcmp(&tbu[i][k], "\\*", 2)) tbu[i][k]   = '/';
         }

      for ( i=rho[0]-1; i>=0; i-- )
        for ( k=lens[i]-2; k>=0; k-- )
          if (!bcmp(&tbu[i][k], "*/", 2))
            {  bcopy(tbu[i]+k+2, tbu[i]+k, lens[i]-k-1);
               goto end_i2;
            }
      end_i2:;

      for ( i=0; i<rho[0]-1; i++ ) puts(tbu[i]);
      printf("%s", tbu[i]);

   }
  else
   {  printf ( "/* ");
      for ( i=0; i<rho[0]; i++ )
        { for ( k=0; k<lens[i]-1; k++ )
           { if (!bcmp(&tbu[i][k], "*/", 2)) tbu[i][k+1] = '\\';
             if (!bcmp(&tbu[i][k], "/*", 2)) tbu[i][k]   = '\\';
           }

          if ( i != rho[0]-1 ) puts(tbu[i]);
          else                 printf("%s", tbu[i]);
        }
      printf (" */");
   }

}

/*
 *
 *  readln2 : reads nl lines from the file corresponding to fd in
 *            tbu[0], tbu[1], ....
 *            rho[0]    will be the effective number of lines read
 *            rho[1]    will be the maximum line length
 *            *lens[i]  will be the length of line i
 *
 *            returns -1 in case of error
 *
 *            buff[rho[0]] receives a NULL
 *
 *  Example :      int rho[2], *lens;
 *                 char **tbu;
 *
 *                 readln2( fd, 1, &tbu, rho, &lens );
 *
 */


#define PAS 1024

#define Max(a,b)  ( ((a) > (b)) ? (a) : (b) )

static char oldt[PAS];
static int l_oldt = 0;

 int readln2( fd, nl, tbu, rho, lens )
   int fd, nl, rho[2], **lens;
   char ***tbu;
{
  char **buff, tamp[PAS], *ext;
  register i, j, k, id, len, ll;
  int nlus, nbext, nc, *lne, lg;

  rho[1] = 0;
  id     = 0;
  nlus    = 0;
  nbext   = 0;


  buff    = (char **) malloc(1);
  lne     = NULL;
  ext     = NULL;

  while ( 1 )
   {  if (l_oldt)
        { bcopy(oldt, tamp, l_oldt);
          for ( i=0, lg=1; i<l_oldt;i++ )
             if ( tamp[i] == '\n' )
               { lg = 0;
                 break;
               }

          if (lg)
            { if (-1 == (nc = read(fd, tamp+l_oldt, PAS-l_oldt)) ) return(-1);
              nc += l_oldt;
            }
          else nc = l_oldt;

          l_oldt = 0;
        }
      else  if ( -1 == (nc = read(fd, tamp, PAS)) ) return(-1);
       if ( nc != 0 )
        {  id = 0;

         BO:for ( i=id; i<nc; i++ )
             if (tamp[i] == '\n')
              { len = nbext + i-id;
                if ( tamp[i] != '\n' ) ++len;
                k = nlus++;

                buff = (char **) alloc_n(buff, (1+nlus) * sizeof(char *) );
                if ( buff == NULL ) return(-1);
                lne  = (int  *) alloc_n(lne , nlus * sizeof(int) );
                if ( lne  == NULL ) return(-1);

                buff[k] = malloc(1+len);
                if ( buff[k] == NULL ) return(-1);

                lne[k] = len;

                for ( j=0; j<nbext; j++ )    buff[k][j] = ext[j];
                for ( j=nbext; j<len; j++ )  buff[k][j] = tamp[id+j-nbext];
                buff[k][len] = '\0';

                nbext = 0;
                free_n(&ext);
                rho[1] = Max( rho[1], len );

                id = i+1;
                if ( nlus == nl ) goto END;
                goto BO;
              }

             if ( id != nc )
              { k = nbext;
                nbext += (ll = nc - id);
                if ( NULL == (ext = alloc_n( ext, nbext )) )return(-1);
                for ( i=0; i<ll; i++ ) ext[k+i] = tamp[i+id];
              }
        }
      else
        { if ( nbext != 0 )
            {   k = nlus++;
                buff = (char **) alloc_n(buff, (1+nlus) * sizeof(char *) );
                if ( buff == NULL ) return(-1);
                lne  = (int  *) alloc_n(lne , nlus * sizeof(int) );
                if ( lne  == NULL ) return(-1);

                buff[k] = malloc(1+nbext);
                if ( buff[k] == NULL ) return(-1);

                lne[k] = nbext;

                for ( j=0; j<nbext; j++ )    buff[k][j] = ext[j];
                buff[k][nbext] = '\0';

                rho[1] = Max( rho[1], nbext );

                nbext = 0;
                free_n(&ext);
                if ( nlus == nl ) goto END;
            }
           break;
        }
   }

END:;

     if ( id < nc )
      { l_oldt = nc-id;
        bcopy(tamp+id, oldt, l_oldt);
      }

   rho[0] = nlus;
   buff[nlus] = NULL;
   *tbu   = buff;
   *lens  = lne;
   return(0);

}


char *alloc_n(pt,nb)
  char *pt;
  int nb;

{
  if ( pt == NULL ) return ( malloc(nb) );
  else              return ( realloc(pt,nb) );
}

void free_n(pt)
  char **pt;
{
  char*free();

  if ( *pt != NULL )
  {  free(*pt);
     *pt = NULL;
  }
}

