RUN_RX - Run an S/REXX program


#include "rexx.h"

int run_rx(args, inst, ss, ret_str, len_ret_str)
  char *args, *inst, **ret_str;
  SAV_COMP **ss;
  int *len_ret_str;



args is a NULL terminated string which starts with a filename related to
an S/REXX program, optionally followed by the argument to be passed to
this program.

inst is a NULL terminated string indicating the S/REXX installation
directory, which must contain a passwds file with a valid password
identifying the cpu on which run_rx will execute. If inst is NULL, run_rx
will attempt to use a REXXHOME environment variable instead. If this case
before using run_rx, the user must issue a command such as
"setenv REXXHOME /home/xed" (C-Shell) or
"export REXXHOME=/home/xed" (Korn Shell).


On WINDOWS systems, inst can optionally contain the name of the .exe
module to be used when the REXX CALL clause is used with OPTION NOLOAD in
effect:

-----------------------------------------------------------------------
#  define INST "C:\\Program Files\\SEDIT\\srexx.new.exe"
  rc = run_rx(args, INST, NULL, NULL, NULL);
-----------------------------------------------------------------------


ss is a pointer to a structure which will be filled with the compiled form
of the args S/REXX program, or a NULL pointer if this save is not to be
performed.

If ss is not a NULL pointer, it must be set to zero before the first
run_rx call.
Subsequent run_rx calls will use the ss compiled form instead of compiling
every time the args program, saving a substantial amount of time. Note
that run_rx will notice if the args program has been externally modified
between two successive calls, and automatically compile it again.

If not a NULL pointer, ret_str will be filled with an allocated NULL
terminated string containing the string passed to the S/REXX EXIT
statement, if any. len_ret_str will contain the ret_str length.

It is the responsibility of the caller to free *ret_str after usage.


Example: A C demo6.c program:


-----------------------------------------------------------------------
#include <stdio.h>
#include "srexx.h"
int call_back(string, len)
  char *string;
{ printf ("call_back: received \"%s\"\n", string);
  return(0);
}
void dsp(exit_value, len_exit_value, rc)
  char *exit_value;
  int len_exit_value, rc;
{ if (exit_value)
   { printf ("demo6: exit_value = \"%s\",  len = %d\n",
             exit_value, len_exit_value);
     free(exit_value);
   }
  printf ("demo6: return code = %d\n", rc);
}
main()
{ char *exit_value;
  SAV_COMP *compiled;
  int len_exit_value, rc;
  env_rx("MyEnv", call_back, 1);
  /*
   * Compiled form not saved
   */
  run_rx("r_demo6", "/home/xed", NULL, NULL, NULL);
  /*
   * Compiled form saved
   */
  compiled = NULL;    /*  **** MANDATORY **** */
  rc = run_rx("r_demo6", "/home/xed", &compiled,
              &exit_value, &len_exit_value);
  dsp(exit_value, len_exit_value, rc);
  /*
   * run_rx() will use the previous compiled form
   */
  rc = run_rx("r_demo6 1", "/home/xed", &compiled,
              &exit_value, &len_exit_value);
  dsp(exit_value, len_exit_value, rc);
  /*
   * run_rx() will recompile r_demo6  because we modify it
   */
  system("touch r_demo6");
  rc = run_rx("r_demo6 2", "/home/xed", &compiled,
              &exit_value, &len_exit_value);
  dsp(exit_value, len_exit_value, rc);
  rc = run_rx("r_demo6 3", "/home/xed", &compiled,
               &exit_value, &len_exit_value);
  dsp(exit_value, len_exit_value, rc);
 }
-----------------------------------------------------------------------


A r_demo6 S/REXX program called by the demo6 program:


-----------------------------------------------------------------------
parse arg what .

say
say '*************************************'
say 'demo6: called with "'what'" argument.'

'Sent to the MYENV callback routine'

select
  when what = 1 then exit
  when what = 2 then exit 12**1.001
  when what = 3 then return 'Exit String'
  otherwise nop
end
-----------------------------------------------------------------------


To compile and execute this program, issue the following commands:

-----------------------------------------------------------------------
C{foo}% cd {install-dir}/demo/demo_sr
C{demo_sr}% make demo6
cc -O -o demo6 demo6.c ../../lib/sun4/libsr.o -I../../include  -lc -lm
9.1u 1.6s 0:12 84% 0+880k 49+67io 82pf+0w
C{demo_sr}% demo6

*************************************
demo6: called with "" argument.
call_back: received "Sent to the MYENV callback routine"

*************************************
demo6: called with "" argument.
call_back: received "Sent to the MYENV callback routine"
demo6: return code = 0

*************************************
demo6: called with "1" argument.
call_back: received "Sent to the MYENV callback routine"
demo6: return code = 0

*************************************
demo6: called with "2" argument.
call_back: received "Sent to the MYENV callback routine"
demo6: exit_value = "12.029856",  len = 9
demo6: return code = 12

*************************************
demo6: called with "3" argument.
call_back: received "Sent to the MYENV callback routine"
demo6: exit_value = "Exit String",  len = 11
demo6: return code = 1165519220
C{demo_sr}%
-----------------------------------------------------------------------

where {install-dir} is the actual S/REXX installation directory.
